Package: furrr
Title: Apply Mapping Functions in Parallel using Futures
Version: 0.4.0
Authors@R: c(
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4777-038X")),
    person("Henrik", "Bengtsson", , "henrikb@braju.com", role = "aut",
           comment = c(ORCID = "0000-0002-7579-5165")),
    person("Matt", "Dancho", , "mdancho@business-science.io", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Implementations of the family of map() functions from 'purrr'
    that can be resolved using any 'future'-supported backend, e.g.
    parallel on the local machine or distributed on a compute cluster.
License: MIT + file LICENSE
URL: https://github.com/futureverse/furrr,
        https://furrr.futureverse.org/
BugReports: https://github.com/futureverse/furrr/issues
Depends: future (>= 1.70.0), R (>= 4.1.0)
Imports: globals (>= 0.19.1), purrr (>= 1.2.1), rlang (>= 1.1.7), vctrs
        (>= 0.7.0)
Suggests: carrier, covr, dplyr (>= 1.1.4), knitr, parallelly (>=
        1.46.1), testthat (>= 3.3.2), tidyselect
Config/Needs/website: progressr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-03-30 17:08:19 UTC; davis
Author: Davis Vaughan [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4777-038X>),
  Henrik Bengtsson [aut] (ORCID: <https://orcid.org/0000-0002-7579-5165>),
  Matt Dancho [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2026-03-31 14:50:02 UTC
