# Generated by roxygen2: do not edit by hand

S3method(future_modify,character)
S3method(future_modify,default)
S3method(future_modify,double)
S3method(future_modify,integer)
S3method(future_modify,logical)
S3method(future_modify,pairlist)
S3method(future_modify_at,character)
S3method(future_modify_at,default)
S3method(future_modify_at,double)
S3method(future_modify_at,integer)
S3method(future_modify_at,logical)
S3method(future_modify_if,character)
S3method(future_modify_if,default)
S3method(future_modify_if,double)
S3method(future_modify_if,integer)
S3method(future_modify_if,logical)
S3method(print,furrr_options)
export(furrr_options)
export(future_imap)
export(future_imap_chr)
export(future_imap_dbl)
export(future_imap_dfc)
export(future_imap_dfr)
export(future_imap_int)
export(future_imap_lgl)
export(future_imap_vec)
export(future_iwalk)
export(future_map)
export(future_map2)
export(future_map2_chr)
export(future_map2_dbl)
export(future_map2_dfc)
export(future_map2_dfr)
export(future_map2_int)
export(future_map2_lgl)
export(future_map2_vec)
export(future_map_at)
export(future_map_chr)
export(future_map_dbl)
export(future_map_dfc)
export(future_map_dfr)
export(future_map_if)
export(future_map_int)
export(future_map_lgl)
export(future_map_vec)
export(future_modify)
export(future_modify_at)
export(future_modify_if)
export(future_pmap)
export(future_pmap_chr)
export(future_pmap_dbl)
export(future_pmap_dfc)
export(future_pmap_dfr)
export(future_pmap_int)
export(future_pmap_lgl)
export(future_pmap_vec)
export(future_pwalk)
export(future_walk)
export(future_walk2)
import(rlang)
importFrom(future,future)
importFrom(parallel,nextRNGStream)
importFrom(parallel,nextRNGSubStream)
importFrom(purrr,map)
