#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import rlang
#' @importFrom purrr map
#' @importFrom future future
#' @importFrom parallel nextRNGSubStream
#' @importFrom parallel nextRNGStream
## usethis namespace: end
NULL
