% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/furrr-package.R
\docType{package}
\name{furrr-package}
\alias{furrr}
\alias{furrr-package}
\title{furrr: Apply Mapping Functions in Parallel using Futures}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Implementations of the family of map() functions from 'purrr' that can be resolved using any 'future'-supported backend, e.g. parallel on the local machine or distributed on a compute cluster.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/futureverse/furrr}
  \item \url{https://furrr.futureverse.org/}
  \item Report bugs at \url{https://github.com/futureverse/furrr/issues}
}

}
\author{
\strong{Maintainer}: Davis Vaughan \email{davis@posit.co} (\href{https://orcid.org/0000-0003-4777-038X}{ORCID})

Authors:
\itemize{
  \item Henrik Bengtsson \email{henrikb@braju.com} (\href{https://orcid.org/0000-0002-7579-5165}{ORCID})
  \item Matt Dancho \email{mdancho@business-science.io}
}

Other contributors:
\itemize{
  \item Posit Software, PBC [copyright holder, funder]
}

}
\keyword{internal}
